% Implementation of |SYSTEM PRIORS|
%
% The interface:
%
% function [log_sp] = |modelname|_spriors(Model, DynareResults, DynareOptions, DynareDataset, DatasetInfo)
%
% Must return the log of the PDF.
%
%
% Contact: mandrle@imf.org


function [log_sp] = ar2_spriors(Model, DynareResults, DynareOptions, DynareDataset, DatasetInfo)

	log_sp = 0;

	% ------------------------------------------------------
	% specify hyper-parameters
	% ------------------------------------------------------

	% sprior hyper for BC freq shares
	hyper1 = 15;
	hyper2 = 5;
	%keyboard
	


	% you can solve the model just for fun... if needed for spriors
	% ------------------------------------------------------
	%[T,R,SteadyState,info,Model,DynareOptions,DynareResults] = dynare_resolve(Model,DynareOptions,DynareResults,'restrict');




	% ------------------------------------------------------
	% for the AR2 model: stationarity + BC variance
	% ------------------------------------------------------

	a1  = Model.params(1); % well, ideally you search by name ;)
	a2  = Model.params(2);
	sig = Model.params(3);

	% stationary if 
	is_stationary = 0;
	if ((a1+a2 < 1) & (a2 - a1 < 1) & (abs(a2)<1) )
		is_stationary = 1;
	end

	if is_stationary == 0, log_sp = Inf; return; end

	% some freq, say the BC freq
	freq  = linspace(0.01,pi,100)';

	Sy = nan(numel(freq),1);
	sel = (freq/(2*pi)>1/32) & (freq/(2*pi) < 1/4); % whatever... 

	for i = 1 : numel(freq)
   	    the_term = 2*pi * (1+ a1^2 + a2^2  + 2*(a1*a2-a1)*cos(freq(i)) - 2*a2*cos(2*freq(i)) );
	    % new_term = 2*pi*(1-a1*exp(-1i*freq(i)) -a2*exp(-1i*2*freq(i)))*(1-a1*exp(1i*freq(i)) -a2*exp(1i*2*freq(i))) ;
	    Sy(i) = sig^2 / the_term;
	end

	% share of variance at freqs
	share = sum(Sy(sel)) / sum(Sy);

	% create the penalty term
	%share_log = log(betapdf(share, hyper1, hyper2)); % gentle nudge..
	share_log  = log(pdf('Beta',share, 16, 4)); % hammer for testing

	% augment the aggregate penalty term
	log_sp = log_sp + share_log;




	% ----------------------------------------------
	%  another sprior....
	% ----------------------------------------------
	another_sprior_log = 0;

		% copute moments, spectra, IRFs, whatever and compare wity
		% your system priors

	log_sp = log_sp + another_sprior_log;


	return




end
